@@ -11,7 +11,7 @@ import unittest
 test_app.py
 websocket - WebSocket client library for Python
 
-Copyright 2022 engn33r
+Copyright 2023 engn33r
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -191,7 +191,6 @@ class WebSocketAppTest(unittest.TestCase):
     # def testOpcodeBinary(self):
     #     """ Test WebSocketApp binary opcode
     #     """
-    #     # The lack of wss:// in the URL below is on purpose
     #     app = ws.WebSocketApp('wss://streaming.vn.teslamotors.com/streaming/')
     #     app.run_forever(ping_interval=2, ping_timeout=1, ping_payload="Ping payload")
 
@@ -311,8 +310,8 @@ class WebSocketAppTest(unittest.TestCase):
         app.run_forever(ping_interval=2, ping_timeout=1, reconnect=3)
 
         self.assertEqual(pong_count, 2)
-        self.assertIsInstance(exc, ValueError)
-        self.assertEqual(str(exc), "Invalid file object: None")
+        self.assertIsInstance(exc, ws.WebSocketTimeoutException)
+        self.assertEqual(str(exc), "ping/pong timed out")
 
 
 if __name__ == "__main__":
